==> GeneR_2.6.0.tar.gz: GeneR/src/complementaire.cc <==
/*! \file complementaire.cc
 *  \brief all functions to make reverse complement
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2006-03-01 14:57:08 lucas>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include <stdio.h>
#include <string>
#include "GeneR_glob.h"
#include "complementaire.h"


extern "C"
{
  void buf_complementaire(int *bufseq);
  void str_complementaire(char **seq);
  void reverse_string(char **str);
}


/* ---------------------------------------------------- */
/*                   complementaire			*/
/* ---------------------------------------------------- */
/** @brief donne le reverse complementaire de la sequence
           place dans le buffer bufseq
	   et la place dans le buffer bufcomp
	   < LC > add 16 Mar 00
    @param bufseq Numero de la sequence
*/
/* ---------------------------------------------------- */

void buf_complementaire(int *bufseq)
{

   int size;
   char *comple; 
   char * seq;
   char * seqComp;
   int comp;
   
   comp = 1;


  
   seq =  myGeneRbuffer(*bufseq,0);

   if (! seq)
     {
       fprintf(stderr,"Empty buffer %d\n",*bufseq);
       *bufseq = -1;
       return;
     }
   
   size =  myGeneRsize(*bufseq,0);


   if(!myGeneRallocBuffer(size,*bufseq,1,0))
     {
       fprintf(stderr,"GeneR.so: allocation error\n");
       *bufseq = -1;
       return;
     }

   seqComp =  myGeneRbuffer(*bufseq,1);


   strcpy(seqComp,seq);
      
   comple = complementaire::compl_init_tab(); 
   
   complementaire::compl_seq(seqComp, comple);

   reverse_string(&seqComp);
	
}

/** ---------------------------------------------------- 
 * \fn str_complementaire	(char **seq)			
 * \brief donne le complementaire d'une sequence	
 * \param seq sequence character to complemente
 * \note must be used with reverse_string
 * ---------------------------------------------------- */
void str_complementaire(char **seq)

{
   
   char *comple ; 
   
   
   if (! *seq)
   	return;
   
   comple = complementaire::compl_init_tab(); 
   
   complementaire::compl_seq(*seq, comple);
  
}



/** \fn reverse_string	(char **str)		
 * @brief  reverse symbol order in string 'str' 	
 */

void reverse_string(char **str)
{
        char *sb, *se, c;

	if (! *str)
	    return;
	    
        sb = *str;
        se = *str + strlen(*str) - 1;

        while(sb <= se) {
           c    = *sb;
          *sb++ = *se;
          *se-- = c;
        }
}



namespace complementaire{


  /** \brief alias to buf_complementaire
   * \param bufseq number of buffer
   */
  void buf_sys_complementaire(int bufseq)
  {
    buf_complementaire(&bufseq);
  }

  /**
   * \fn compl_seq(char *s, char *co) 
   * \brief  cree le complementaire d'une sequence 
   */
  void compl_seq(char *s, char *co)
  {
    
    s--;
  
    while(*(++s)) {
      *s = co[(int)(*s)];
    }

    return;

  }



  /**
   * \fn compl_init_tab(void)
   * \brief cree un tableau acgt
   */
  char *compl_init_tab(void)
  {
    int i;
    static char co[256];
    
    for(i = 0; i < 256; i++) {
      co[i] = 'N';
    }
   
    co[(int)'A'] = 'T';
    co[(int)'G'] = 'C';
    co[(int)'C'] = 'G';
    co[(int)'T'] = 'A';
    co[(int)'U'] = 'A';
    co[(int)'a'] = 't';
    co[(int)'g'] = 'c';
    co[(int)'c'] = 'g';
    co[(int)'t'] = 'a';
    co[(int)'u'] = 'a';
   
    return co;
  }
}

==> GeneR_2.10.0.tar.gz: GeneR/src/complementaire.cc <==
/*! \file complementaire.cc
 *  \brief all functions to make reverse complement
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2007-10-12 18:35:33 antoine>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include <stdio.h>
#include <string>
#include "GeneR_glob.h"
#include "complementaire.h"


extern "C"
{
  void buf_complementaire(int *bufseq);
  void str_complementaire(char **seq);
  void reverse_string(char **str);
}


/* ---------------------------------------------------- */
/*                   complementaire			*/
/* ---------------------------------------------------- */
/** @brief donne le reverse complementaire de la sequence
           place dans le buffer bufseq
	   et la place dans le buffer bufcomp
	   < LC > add 16 Mar 00
    @param bufseq Numero de la sequence
*/
/* ---------------------------------------------------- */

void buf_complementaire(int *bufseq)
{

   int size;
   char *comple; 
   char * seq;
   char * seqComp;
   int comp;
   
   comp = 1;


  
   seq =  GeneR_glob::instance()->buffer(*bufseq,0);

   if (! seq)
     {
       fprintf(stderr,"Empty buffer %d\n",*bufseq);
       *bufseq = -1;
       return;
     }
   
   size =  GeneR_glob::instance()->size(*bufseq,0);


   if(!GeneR_glob::instance()->allocBuffer(size,*bufseq,1,0))
     {
       fprintf(stderr,"GeneR.so: allocation error\n");
       *bufseq = -1;
       return;
     }

   seqComp =  GeneR_glob::instance()->buffer(*bufseq,1);


   strcpy(seqComp,seq);
      
   comple = complementaire::compl_init_tab(); 
   
   complementaire::compl_seq(seqComp, comple);

   reverse_string(&seqComp);
	
}

/** ---------------------------------------------------- 
 * \fn str_complementaire	(char **seq)			
 * \brief donne le complementaire d'une sequence	
 * \param seq sequence character to complemente
 * \note must be used with reverse_string
 * ---------------------------------------------------- */
void str_complementaire(char **seq)

{
   
   char *comple ; 
   
   
   if (! *seq)
   	return;
   
   comple = complementaire::compl_init_tab(); 
   
   complementaire::compl_seq(*seq, comple);
  
}



/** \fn reverse_string	(char **str)		
 * @brief  reverse symbol order in string 'str' 	
 */

void reverse_string(char **str)
{
        char *sb, *se, c;

	if (! *str)
	    return;
	    
        sb = *str;
        se = *str + strlen(*str) - 1;

        while(sb <= se) {
           c    = *sb;
          *sb++ = *se;
          *se-- = c;
        }
}



namespace complementaire{


  /** \brief alias to buf_complementaire
   * \param bufseq number of buffer
   */
  void buf_sys_complementaire(int bufseq)
  {
    buf_complementaire(&bufseq);
  }

  /**
   * \fn compl_seq(char *s, char *co) 
   * \brief  cree le complementaire d'une sequence 
   */
  void compl_seq(char *s, char *co)
  {
    
    s--;
  
    while(*(++s)) {
      *s = co[(int)(*s)];
    }

    return;

  }



  /**
   * \fn compl_init_tab(void)
   * \brief cree un tableau acgt
   */
  char *compl_init_tab(void)
  {
    int i;
    static char co[256];
    
    for(i = 0; i < 256; i++) {
      co[i] = 'N';
    }
   
    co[(int)'A'] = 'T';
    co[(int)'G'] = 'C';
    co[(int)'C'] = 'G';
    co[(int)'T'] = 'A';
    co[(int)'U'] = 'A';
    co[(int)'a'] = 't';
    co[(int)'g'] = 'c';
    co[(int)'c'] = 'g';
    co[(int)'t'] = 'a';
    co[(int)'u'] = 'a';
   
    return co;
  }
}
